@@ -61,7 +61,6 @@ def save_hdf_dataset(ds, fname, verbose=True):
         data = hdf.create_group('data')
         for num, chunk in enumerate(ds.data):
             spec = data.create_group('spec%i' % (num+1))
-            spec.attrs['filename'] = ds.data_filenames[num]
             spec.attrs['res'] = chunk['res']
             spec.attrs['norm'] = chunk['norm']
             spec.attrs['nsub'] = chunk['nsub']
@@ -106,9 +105,8 @@ def save_hdf_dataset(ds, fname, verbose=True):
         fine_lines = hdf.create_group('fine_lines')
         if hasattr(ds, 'fine_lines'):
             for ground_state, lines in ds.fine_lines.items():
-                # line_array = np.array(lines, dtype='str')
-                line_array = [s.encode("ascii", "ignore") for s in fine_lines]
-                fine_lines.create_dataset(str(ground_state), data=line_array)
+                line_array = np.array(lines, dtype='str')
+                fine_lines.create_dataset(ground_state, data=line_array)
 
         # .components:
         components = hdf.create_group('components')
@@ -170,11 +168,6 @@ def load_dataset_from_hdf(fname):
         # Load .data:
         data = hdf['data']
         for chunk in data.values():
-            # For backward compatibility:
-            if 'filename' in chunk.attrs.keys():
-                filename = chunk.attrs['filename']
-            else:
-                filename = ''
             res = chunk.attrs['res']
             norm = chunk.attrs['norm']
             if 'nsub' in chunk.attrs.keys():
@@ -191,7 +184,7 @@ def load_dataset_from_hdf(fname):
 
             ds.add_data(wl, flux, res,
                         err=error, normalized=norm, nsub=nsub,
-                        mask=mask, filename=filename)
+                        mask=mask)
 
         # Load .regions:
         # --- this will be deprecated in later versions
@@ -281,8 +274,7 @@ def load_dataset_from_hdf(fname):
             fine_lines = hdf['fine_lines']
             if len(fine_lines) > 0:
                 for ground_state, line_tags in fine_lines.items():
-                    unicode_list = [s.decode('utf-8') for s in line_tags]
-                    ds.fine_lines[ground_state] = unicode_list
+                    ds.fine_lines[ground_state] = list(line_tags)
 
         # Load .components:
         components = hdf['components']
@@ -292,16 +284,14 @@ def load_dataset_from_hdf(fname):
 
         for ion, comps in components.items():
             ds.components[ion] = list()
-            N_comps = len(comps)
-            if N_comps > 0:
-                for n in range(N_comps):
-                    pointer = '/components/%s/comp%i' % (ion, n+1)
-                    comp = hdf[pointer]
+            if len(comps) > 0:
+                for n, comp in enumerate(comps.values()):
                     if 'best_fit' in hdf:
                         # If 'best_fit' exists, use the best-fit values.
                         # The naming for 'best_fit' and 'components' is parallel
                         # so one variable in components can easily be identified
                         # in the best_fit data group by replacing the path:
+                        pointer = comp.name
                         fit_pointer = pointer.replace('components', 'best_fit')
                         z = hdf[fit_pointer].attrs['z']
                         z_err = hdf[fit_pointer].attrs['z_err']
@@ -344,7 +334,7 @@ def load_dataset_from_hdf(fname):
                         ds.best_fit[N_name].stderr = logN_err
 
         if 'best_fit' in hdf:
-            # Now the components have been defined in `ds`, so I can use them for the loop
+            # Now the components have been defined in ds, so I can use them for the loop
             # to set the parameter ties:
             for ion, comps in ds.components.items():
                 for n, comp in enumerate(comps):
